﻿
/*
    **************************
    This script creates an AggFraction object to do the heavy lifting and a user interface to capture data.
    User selects the analysis area in Photoshop and runs the script. The script cuts smaller images out
    of the selection, draws a cursor over the middle of the image and prompts the user to identify whether
    the cursor is over aggregate or not. Aggregate fraction is the number of affirmative responses divided by 
    the total number of sample images.

    **************************
*/


//  set the target application to photoshop
#target photoshop

//  include the BubbleCounter script or none of this beautiful work will
#include "BC_Classes/AggFraction.jsxinc"

//  turn off dialogs, seems to speed processing?
app.displayDialogs = DialogModes.NO;
//  get the present ruler units so that the app can be set back to its original state
var uStrtRulerUnits = app.preferences.rulerUnits;
//  set the ruler units to pixels
app.preferences.rulerUnits = Units.PIXELS;

var minSampleCount = 500; // set to the minimum sample count required for analysis

var af = new AggregateFraction();
af.documentOriginal = app.activeDocument; // tells the agg fraction calculator what image we're working with
af.saveSampleImages = false; // set to true to save a copy of every sample image
af.sampleImageSize = 300; // the sample will be a square having this dimension in pixels. 300 pixels is a good place to start.

// setup the user interface
var dlg = new Window('dialog', 'Aggregate fraction');
// following closes the sample document in Photoshop when the dialog is closed
dlg.onClose = function()
    {
        af.closeDocumentNoSave(af.documentSample);
    }

// initialize the agg fraction object
var cont = af.Initialize();

// make sure the sample area will supply the minimum number of samples specified
if (cont && af.samplesAvailable < minSampleCount) cont = confirm("The selected area will yield a total of " + af.samplesAvailable + " samples, which is less than the " + minSampleCount + " minimum required.\n\nContinue?");

// off to the races
if (cont)
    {
        af.copySampleImage(); // required to generate the first sample image
        af.saveSampleImage(); // required to display the first sample image in the dialog box

        // add the image control
        var img = dlg.add('Image', [0, 0, af.sampleImageSize, af.sampleImageSize], af.currentSamplePath); //[left, top, right, bottom] =  bounds

        var cellWidth = Math.round(af.sampleImageSize / 2) - 25;
        // show a little information about what's happening
        dlg.grpInfo = dlg.add('panel');
        dlg.grpInfo.orientation = 'row';
        dlg.grpInfo.preferredSize = [af.sampleImageSize, -1];
        dlg.grpInfo.alignChildren =  ['center', 'bottom'];
        dlg.grpInfo.grpSample = dlg.grpInfo.add('group');
        dlg.grpInfo.grpSample.preferredSize = [cellWidth, -1];
        dlg.grpInfo.grpSample.alignChildren = 'center';
        dlg.grpInfo.grpSample.orientation = 'column';
        // the current sample number
        dlg.grpInfo.grpSample.st = dlg.grpInfo.grpSample.add('StaticText', undefined, 'Sample');
        var stSample = dlg.grpInfo.grpSample.add('StaticText', undefined, '0000'); // have to initialize with some text otherwise it stupidly sets the width to one character
        // the aggregate fraction
        dlg.grpInfo.grpFracAgg = dlg.grpInfo.add('group');
        dlg.grpInfo.grpFracAgg.alignChildren = 'center';
        dlg.grpInfo.grpFracAgg.orientation = 'column';
        dlg.grpInfo.grpFracAgg.preferredSize = [cellWidth, -1];
        dlg.grpInfo.grpFracAgg.st01 = dlg.grpInfo.grpFracAgg.add('StaticText', undefined, 'Aggregate Fraction');
        var stFracAgg = dlg.grpInfo.grpFracAgg.add('StaticText', undefined, 'Not available');

        /* following was for debugging sample area selection
        dlg.grpInfo.grpRow = dlg.grpInfo.add('group');
        //dlg.grpInfo.grpRow.preferredSize = [cellWidth, cellHeight];
        dlg.grpInfo.grpRow.alignChildren = ['center', 'center'];
        dlg.grpInfo.grpRow.orientation = 'column';
        dlg.grpInfo.grpRow.stTitle02 = dlg.grpInfo.grpRow.add('StaticText', undefined, 'Row');
        dlg.grpInfo.grpRow.stTitle02.preferredSize = [cellWidth, -1];
        var stRow = dlg.grpInfo.grpRow.add('StaticText', undefined, af.currentRow + ' of ' + af.rows);
        stRow.preferredSize = [cellWidth, -1];

        dlg.grpInfo.grpColumn = dlg.grpInfo.add('group');
        //dlg.grpInfo.grpColumn.preferredSize = [cellWidth, cellHeight];
        dlg.grpInfo.grpColumn.alignChildren = ['center', 'center'];
        dlg.grpInfo.grpColumn.orientation = 'column';
        dlg.grpInfo.grpColumn.stTitle03 = dlg.grpInfo.grpColumn.add('StaticText', undefined, 'Column');
        dlg.grpInfo.grpColumn.stTitle03.preferredSize = [cellWidth, -1];
        var stColumn = dlg.grpInfo.grpColumn.add('StaticText', undefined, af.currentColumn + ' of ' + af.columns);
        stColumn.preferredSize = [cellWidth, -1];

        //add results row
        dlg.grpResults = dlg.add('panel');
        //dlg.grpResults.orientation = 'row';
        dlg.grpResults.alignChildren =  ['left', 'top'];
        */

        //add the button group
        dlg.buttons = dlg.add('group');
        dlg.buttons.orientation = 'column';
        dlg.buttons.alignChildren = 'center';
        dlg.buttons.agg = dlg.buttons.add('panel', undefined,  'Is the cross on aggregate?');
        dlg.buttons.agg.orientation = 'row';
        dlg.buttons.agg.preferredSize = [af.sampleImageSize, -1];
        var btnAggYes = dlg.buttons.agg.add('button', undefined, 'Yes');
        var btnAggNo = dlg.buttons.agg.add('button', undefined, 'No');

        dlg.buttons.other = dlg.add('group');
        dlg.buttons.other.orientation = 'row';
        var btnReport = dlg.buttons.other.add('button', undefined, 'Report');
        btnReport.enabled = false;
        var btnStats = dlg.buttons.other.add('button', undefined, 'Stats');
        btnStats.enabled = false;
        var btnQuit = dlg.buttons.other.add('button', undefined, 'Quit');
        btnQuit.enabled = false;

        //setup button events
        btnAggYes.onClick = function()
            {
                advance(true);
            }

        btnAggNo.onClick = function()
            {
                advance(false);
            }

        btnReport.onClick = function() // writes a report file with data to current sample
            {
                af.writeReport();
                alert(af.filenameReport + "\n\nwas written to\n\n" + af.path);
            }

        btnStats.onClick = function() // generates a prompt with results and statistical data
            {
                displayStats(false);
            }

        btnQuit.onClick = function() // will not activate until the minimum number of samples is processed. Writes the report and displays the stats.
            {
                af.writeReport();
                displayStats(true);
                dlg.close(1);
            }
        // setup the key press event so users can select Y or N on the keyboard = lots faster
        dlg.addEventListener("keydown", function(kd){evalKey(kd)});

        dlg.show();
        
    }

// event that fires upon a keypress
function evalKey(k)
    {
        if (k.keyName.toLowerCase() == "y") advance(true);
        if (k.keyName.toLowerCase() == "n") advance(false);
/*      for learning purposes:
        $.writeln (k.keyName);
        $.writeln (k.keyIdentifier);
        $.writeln (k.shiftKey ? "Shift pressed" : "Shift not pressed");
        $.writeln (k.altKey ? "Alt pressed" : "Alt not pressed");
        $.writeln (k.ctrlKey ? "Ctrl pressed" : "Ctrl not pressed");
        */
    }

function displayStats(isFinal)
    {
        var stats = af.stats();
        var msgStats = "Mean aggregate fraction: " + Math.round(stats.meanAF * 10000) / 10000;
        msgStats += "\nStandard deviation: " + Math.round(stats.stdDev * 10000) / 10000;
        msgStats += "\nRelative standard deviation: " + Math.round(stats.stdDev * 10000 / stats.meanAF) / 100 + "%";
        if (isFinal) msgStats += "\n\n" + af.filenameReport + "\n\nwas written to\n\n" + af.path;
        alert(msgStats);
    }

function advance(isAggregate) // stores current results to array and advances sample image.
    {
        af.addResult(isAggregate);
        af.purgePreviousSample();
        if (af.incrementSampleBounds())
            {
                af.copySampleImage();
                af.saveSampleImage();
                img.image = af.currentSamplePath;
                stSample.text =  af.countSample;
        //        stRow.text = af.currentRow + ' of ' + af.rows; // for tracking what it was doing
        //        stColumn.text = af.currentColumn + ' of ' + af.columns;
                stFracAgg.text = Math.round(af.fracAggregate * 10000) / 10000;
                if (af.countSample > minSampleCount) btnQuit.enabled = true;
                if (af.countSample > 10)
                    {
                        btnReport.enabled = true;
                        btnStats.enabled = true;
                    }
            }
        else
            {
                // we're all done!
                af.writeReport();
                displayStats(true);
                dlg.close(1);
            }
    }

app.preferences.rulerUnits = uStrtRulerUnits;