﻿/*
*******************************    

    Creates a traverse composite with more than the usual number of traverses and then processes groups of the traverse lines
    to create an analysis by depth.
    It will process only files for which a complete property file and original image exist.
    Requires that images be analyzed using the latest version of BC as the property file contents have changed.
    New composite image, tabbed text tif image, results and exceptions files are stored in a user definable location (nameFolderResults).
    The script is intended to run even if it cannot get all of the data it needs from the files. Most exceptions will be listed in the exceptions file
    in the report folder. If there is no exception file either there were no errors or the script exited with an error that it could not catch.
    Results are are presented by their pixel height range.

*******************************    
*/
//  set the target application to photoshop
#target photoshop

//  include the BubbleCounter script
#include "BC_Classes/BubbleCounter.jsxinc"
#include "BC_Classes/PropertyFile.jsxinc"

//  set the following to the desired values
var traverses = 200; // total number of travers lines to extract from the image
var groupTraverses = 5; // the number of traverses to analyze at a time
var nameFolderResults = "/depth01/"; // folder name where results will be saved. If subsequent runs are made without changing this, contents will be overwritten


/*
    choices for the following:
        "default" = what it finds in BC defaults
        "props" = what it finds in the image's property file
        values you enter

*/

var thresholdAirContent = "props";
var thresholdVoidFreq = "props";
var airCorrectionFactor = "props";

var analysisMethod = "props"; // similar to above, but isn't found in BC defaults



var sPath = new Folder().selectDlg ("Folder containing processed images");

if (sPath != undefined)
    {
        // setup the results folder
        var folderResults = new Folder(sPath + nameFolderResults);
        if (!folderResults.exists) folderResults.create();

        //  get all the property files in the folder
        var asFiles = new Folder(sPath).getFiles("*_props.csv");
        var countPropertyFiles = asFiles.length;
        var pf;
        var df;
        var bc;
        for (var i = 0; i < countPropertyFiles; i++)
            {
                pf = new PropertyFile(asFiles[i].path, asFiles[i].name);
                pf.read();
                df = File(sPath + "/" + pf.fileNameOriginalImage);
               // make sure we have everything we need
                var fileNameOriginalImage = pf.fileNameOriginalImage;
                var boundsAnalysis = pf.boundsAnalysis;
                var resolution = pf.resolution;
                var intensityBlackMax = pf.intensityBlackMax;
                var intensityWhiteMax = pf.intensityWhiteMax;
                var stepHorizontal = pf.stepHorizontal;
                
                var exception = "";
                if (fileNameOriginalImage == undefined || fileNameOriginalImage == "undefined") exception += "no fileNameOriginalImage\n";
                if (boundsAnalysis == undefined || boundsAnalysis == "undefined") exception += "no boundsAnalysis\n";
                if (resolution == undefined || isNaN(resolution)) exception += "no resolution\n";
                if (intensityBlackMax == undefined || intensityBlackMax == "undefined") exception += "no intensityBlackMax\n";
                if (intensityWhiteMax == undefined || intensityWhiteMax == "undefined") exception += "no intensityWhiteMax\n";
                if (stepHorizontal == undefined || stepHorizontal == "undefined") exception += "no stepHorizontal\n";
                if (!df.exists) exception += "Original image file not found\n";
                
                if (exception == "")
                    {
                        open(df);
                        // select the analysis area
                        app.activeDocument.selection.select(boundsAnalysis);
                        // instantiate a new BC
                        bc = new BubbleCounter(); 
                        // setup BC
                        bc.documentOriginal = app.activeDocument;
                        bc.boundsAnalysis = bc.documentOriginal.selection.bounds;
                        bc.micronsPerPixel = 25400 / resolution;
                        bc.path = sPath + nameFolderResults;
                        bc.fileNameOriginalImage = fileNameOriginalImage;
                        bc.fileNameProperties = asFiles[i].name;
                        bc.fileNameCompositeImage = bc.fileNameOriginalImage.replace(".tif", "_comp.tif");
                        bc.fileNameTabDelimited = bc.fileNameOriginalImage.replace(".tif", "_comp.txt");
                        bc.resolution = resolution;
                        bc.intensityBlackMax = intensityBlackMax;
                        bc.intensityWhiteMax = intensityWhiteMax;
                        bc.pasteToAgg = pf.pasteToAgg;
                        bc.pasteVolume = pf.pasteVolume;
                        bc.aggVolume = pf.aggVolume;
                        bc.stepHorizontal = stepHorizontal;
                        bc.traverses = traverses;
                         switch (thresholdAirContent)
                            {
                                case "props" :
                                    bc.thresholdAirContent = pf.thresholdAirContent;
                                    break;
                                case "default" :
                                    bc.thresholdAirContent = bcThresholdAirContent;
                                    break;
                                default :
                                    if (isNaN(thresholdAirContent))
                                        {
                                            // I gues we're screwed
                                            alert("thresholdAirContent must be numeric, unable to continue...");
                                            i = 10000;
                                        }
                                    else
                                        {
                                            bc.thresholdAirContent = thresholdAirContent;
                                        }
                                    break;
                            }
                         switch (thresholdVoidFreq)
                            {
                                case "props" :
                                    bc.thresholdVoidFreq = pf.thresholdVoidFreq;
                                    break;
                                case "default" :
                                    bc.thresholdVoidFreq = bcThresholdVoidFreq;
                                    break;
                                default :
                                    if (isNaN(thresholdVoidFreq))
                                        {
                                            // I gues we're screwed
                                            alert("thresholdVoidFreq must be numeric, unable to continue...");
                                            i = 10000;
                                        }
                                    else
                                        {
                                            bc.thresholdVoidFreq = thresholdVoidFreq;
                                        }
                                    break;
                            }
                         switch (airCorrectionFactor)
                            {
                                case "props" :
                                    bc.airCorrectionFactor = pf.airCorrectionFactor;
                                    break;
                                case "default" :
                                    bc.airCorrectionFactor = bcAirCorrection;
                                    break;
                                default :
                                    if (isNaN(airCorrectionFactor))
                                        {
                                            // I gues we're screwed
                                            alert("airCorrectionFactor must be numeric, unable to continue...");
                                            i = 10000;
                                        }
                                    else
                                        {
                                            bc.airCorrectionFactor = airCorrectionFactor;
                                        }
                                    break;
                            }
                         switch (analysisMethod)
                            {
                                case "props" :
                                    bc.analysisMethod = pf.analysisMethod;
                                    break;
                                default :
                                    if (isNaN(analysisMethod))
                                        {
                                            // I gues we're screwed
                                            alert("analysisMethod must be numeric, unable to continue...");
                                            i = 10000;
                                        }
                                    else
                                        {
                                            bc.analysisMethod = analysisMethod;
                                        }
                                    break;
                            }
                        // time to do the work
                        bc.setDocumentLevels(bc.documentOriginal);
                        bc.createTraverseComposite();
                        bc.saveDocumentAsTif(bc.documentComposite, bc.fileNameCompositeImage);
                        bc.saveDocumentAsText(bc.documentComposite, bc.fileNameTabDelimited);
                        bc.closeDocumentNoSave(bc.documentComposite);
                        bc.closeDocumentNoSave(bc.documentOriginal);
                        // setup the results report
                        var fResults = new File(sPath + nameFolderResults + pf.fileNameOriginalImage.replace(".tif", "_depth.csv"));
                        with (fResults)
                            {
                                open("w");
                                writeln("Analysis of," + pf.fileNameOriginalImage);
                                writeln("traverses," + traverses);
                                writeln("groupTraverses," + groupTraverses);
                                writeln("analysisMethod," + bc.analysisMethod);
                                writeln("Traverse Y Start,Traverse Y End,Traverse Length (mm),Length Through Air (mm),Intercepts,Air Content,Void Freq,Spacing Factor");
                                close();
                            }
                        // the composite is made; run the analysis on select groups of lines grouped per groupTraverses value
                        bc.readCompositeText();
                        //save the array to a local variable
                        var compIntensity = bc.traverseIntensityArray;
                        // process a group of traverses
                        var iTraverses = 0;
                        while (iTraverses < traverses)
                            {
                                bc.traverseIntensityArray = new Array(groupTraverses);
                                for (var j = 0; j < groupTraverses; j++)
                                    {
                                        bc.traverseIntensityArray[j] = compIntensity[iTraverses];
                                        iTraverses++;
                                    }
                                // process the traverses and save the results to a file
                                bc.processTraversesVF();
                                bc.processTraversesAC();
                                bc.processTraversesAvg();
                                bc.applyAirCorrection();
                                bc.calculate();
                                // stuff the results in the results file
                                with (fResults)
                                    {
                                        open("a");
                                        if (iTraverses < traverses) writeln(bc.boundsTraverses[iTraverses - groupTraverses + 1][1][1] + "," + bc.boundsTraverses[iTraverses][1][1] + "," + bc.traverseLengthmm + "," + bc.airLengthmm + "," + bc.intercepts + "," + bc.airContent + "," + bc.voidFreq + "," + bc.spacingFactor);
                                        else writeln(bc.boundsTraverses[iTraverses - groupTraverses + 1][1][1] + "," + bc.boundsTraverses[traverses-1][1][1] + "," + bc.traverseLengthmm + "," + bc.airLengthmm + "," + bc.intercepts + "," + bc.airContent + "," + bc.voidFreq + "," + bc.spacingFactor);
                                        close();
                                    }
                            }
                    }
                else
                    {
                        // write to the exception file
                        var fExceptions = new File(sPath + nameFolderResults + "exceptions.txt");
                        if (fExceptions.exists) fExceptions.open("a");
                        else fExceptions.open("w");
                        with (fExceptions)
                            {
                                writeln(asFiles[i].name + " reports:");
                                writeln(exception);
                                close();
                            }
                    }
            } 
    }
